#!/usr/bin/env node

require('colors')

const minimist = require('minimist')

const sendError = require('../lib/error')
const login = require('../lib/login/login')

const discordListener = require('../lib/listeners/discord')
const messengerListener = require('../lib/listeners/messenger')
const getConfig = require('../lib/config/getConfig')

var args = minimist(process.argv.slice(2))

if (args.h || args.help) {
  console.log(`
Miscord v${require('../package.json').version}

Usage: miscord
  --help    [-h]              ${'shows this message'.cyan}
  --version [-v]              ${'shows version'.cyan}
  --config  [-c] configPath   ${'reads config from custom path'.cyan}
  --token   [-t] discordToken ${'Discord token for interactive setup (needs userid)'.cyan}
  --userid  [-u] userID       ${'your user ID (needed for interactive setup)'.cyan}

Example:
  miscord -c /path/to/config.json
  miscord -c D:\\Miscord\\config.json
  miscord -t very.long.token -u 312625089143373824
  `)
  process.exit(0)
}

if (args.v || args.version) {
  console.log(require('../package.json').version)
  process.exit(0)
}

require('../lib/logger.js')(args.c || args.config)

getConfig(args.c || args.config, args.t || args.token).then(login).then(config => {
  // when got a discord message
  config.discord.client.on('message', message => discordListener({config, message}))

  // when got a messenger message
  config.messenger.stopListening = config.messenger.client.listen((err, message) => messengerListener({config, err, message}))
}).catch(err => sendError('bin', err))
